/************************************************************************
 *	        	Bezier Curve Test Program V2                    *
 *                   (c) 3/87 by Christopher Johnson                    *
 *									*
 *	This program is public domain, and may be copied, distributed,	*
 *		modified, cussed at and pored over to your heart's 	*
 *		content.  The routine "bezier()" could be useful in	*
 *		programs of your own that could use Bezier curves.	*
 *		The algorithm was presented in the December, 1986	*
 *		issue of BYTE magazine in an article entitled   	*
 *		"Free-Form Curves on Your Micro", written by Steve	*
 *		Enns.  The method is also described in a *very* good    *
 *		book on computer graphics entitled "Fundamentals of 	*
 *		Interactive Computer Graphics" by James D. Foley and	*
 *		Andries Van Dam (c. 1982 by Addison-Wesley).		*
 ************************************************************************/

/* This include file is my own huge, all-encompasing library file.  It contains
	the includes for the VDI, AES, OS and such straight from the ATARI
	Developers Kit.  Simply include the files that have the calls for the
	VDI and such for your compiler
*/

#include "\lib\proteus.h"

#include "\lib\stdio.h"

/*---=== GEM VDI Definitions ===---*/

int gl_hchar,gl_wchar,gl_wbox,gl_hbox,phys_handle,handle,;
int contrl[12],intin[128],ptsin[128],intout[128],ptsout[128];
int work_in[11],work_out[57],pxyarray[100];


/* ---=== BEZIER Program Definitions ===--- */

float xc[5];	/*	x-coordinate of control points	                      */
float yc[5];	/*	y-coordinate of control points                        */
float nbs;	/*	resolution (increment) for bezier routine             */
float x,y;	/*	x-coordinate of bezier curve calculated point         */
float t;	/*	y-coordinate of bezier curve calculated point         */

int xx,yy;	/*	storage of mouse location #1 (x & y coord.)   	      */
int xx1,yy1;	/*	storage of mouse location #2 (x & y coord.)   	      */
int count;	/*	index for accessing control points from xc array      */
int i,i2;	/*	general purpose index variables                       */
int junk;	/*	dummy variable                                        */
int cx1,cy1;	/* 	distance between mouse and control pt. (x & y coord.) */


/***************************************************************************
 *			Initialize Workstation                             *
 ***************************************************************************/

init()
{
	int i;

	appl_init();
	phys_handle=graf_handle(&gl_wchar,&gl_hchar,&gl_wbox,&gl_hbox);
	for(i=0;i<10;work_in[i++]=1);
	work_in[10]=2;
	handle=phys_handle;
	v_opnvwk(work_in,&handle,work_out);
}


/************************************************************************
 *            Bezier Curve Calculation and Drawing Routine              *
 *	Translated into C from algorithm published in the December   	*
 *	1986 issue of BYTE magazine ("Free-Form Curves on Your Micro",  *
 *	by Steve Enns, page 225)					*
 *                                                                      *
 *									*
 *    Local Variables:							*	
 *									*	
 *	float t2	value of t squared 				*
 *	float t3	value of t cubed				*
 *	float nc1	polynomial component 1				*
 *	float nc2	polynomial component 2				*
 *	float nc3	polynomial component 3				*
 *	float nc4	polynomial component 4				*
 ************************************************************************/

bezier()
{

	float t2,t3,nc1,nc2,nc3,nc4;


		count=0;               /* Set index to first control pt. */

		for(t=0.0;t<=1+nbs;t+=nbs)      /* nbs is the step size  */
		{
			t2=t*t;			/* Calculate polynomials */
			t3=t2*t;
			nc1=1-3*t+3*t2-t3;
			nc2=3*t3-6*t2+3*t;
			nc3=3*t2-3*t3;
			nc4=t3;

         			/* Combine polynomials with control pts. */
  
			x=nc1*xc[0]+nc2*xc[1]+nc3*xc[2]+nc4*xc[3];
			y=nc1*yc[0]+nc2*yc[1]+nc3*yc[2]+nc4*yc[3];
					
			/* Put integer values of Bezier Curve pts. into
			   array for Polyline routine to draw curve      */
		
			pxyarray[count++]=(int) x;
			pxyarray[count++]=(int) y;
		}

	count/=2;       /* No. points equals half the no. of coordinates */
	v_pline(handle,count,pxyarray);	    /* Draw the lines from table */
}

/************************************************************************
 *                      Draws Boxes around selected points              *
 *									*
 *	This routine draws a box around the control point number "i"	*
 *		in the color specified by "color"		        *
 ************************************************************************/

draw_box(i,color)
	int i,color;

{

	vsl_color(handle,color);	        /* Set color as specified   */
	
		pxyarray[0]=(int) xc[i]-5;	/* Put the coordinates of a */
		pxyarray[1]=(int) yc[i]-5;	/*   box around the control */
		pxyarray[2]=(int) xc[i]+5;	/*   point into the table   */
		pxyarray[3]=pxyarray[1];	/*   to draw lines around   */
		pxyarray[4]=pxyarray[2];	/*   the point.		    */
		pxyarray[5]=(int) yc[i]+5;
		pxyarray[6]=pxyarray[0];
		pxyarray[7]=pxyarray[5];
		pxyarray[8]=pxyarray[0];
		pxyarray[9]=pxyarray[1];
		v_pline(handle,5,pxyarray);	/* Then actually draw box   */

}

		
/************************************************************************
 *                  Blanks area around Control Point                    *
 *									*
 *	This routine blanks an area (16 x 16 pixels) around the 	*
 *	  control point specified by "i"				*
 ************************************************************************/

blank(i)
	int i;
{

	pxyarray[0]=(int) xc[i]-8;	/*	Set up coordinates of the */
	pxyarray[1]=(int) yc[i]-8;	/* 	top left corner of box    */
	pxyarray[2]=16;			/*	as well as the width &	  */
	pxyarray[3]=16;			/*	height			  */
	vsf_interior(handle,1);		
	vsf_color(handle,0);		/* 	Set for solid black box   */
	v_bar(handle,pxyarray);		/* 	  and draw it...	  */
}

/************************************************************************
 *                      Draw Lines                                      *
 *									*
 *	This routine draws the lines connecting the control points in	*
 *	  the color specified by "color".  The parameter "i" specifies  *
 *	  whether the area around a control point should be "cleaned    *
 *	  up" by passing the positive index of the control point.       *
 *	  For no cleanup, a negative value for "i" is passed.		*
 ************************************************************************/

draw_line(i,color)
	int i,color;
{
	vsl_color(handle,color);        	/* Set up the color           */


		pxyarray[0]=(int) xc[0]; 	/* Endpoints for lines that   */
		pxyarray[1]=(int) yc[0];	/*   connect all four control */
		pxyarray[2]=(int) xc[1];	/*   points...                */
		pxyarray[3]=(int) yc[1];
		pxyarray[4]=(int) xc[3];
		pxyarray[5]=(int) yc[3];
		pxyarray[6]=(int) xc[2];
		pxyarray[7]=(int) yc[2];
		pxyarray[8]=pxyarray[0];
		pxyarray[9]=pxyarray[1];
		
		v_pline(handle,5,pxyarray);	/* Draw the "convex hull"     */

		if(i>=0)			/* If i is positive, blank    */
			blank(i);
}


main()
{

	init();				/* Initialize a workstation          */

	graf_mouse(M_OFF,0x0L);		/* Shut off mousie	             */

	nbs=0.05;			/* Set resolution to high and        */
	count=0;			/*  reset the index		     */

	pxyarray[0]=0;			/* Clear the screen        	     */
	pxyarray[1]=0;
	pxyarray[2]=639;
	pxyarray[3]=199;
	vsf_interior(handle,1);	
	vsf_color(handle,0);
	v_bar(handle,pxyarray);

	xc[0]=50; 			/* Initialize the control points     */
	yc[0]=100;
	xc[1]=320;
	yc[1]=25;
	xc[2]=320;
	yc[2]=175;
	xc[3]=600;
	yc[3]=100;

	cx1=(int) xc[0];		/* Give these a value to start        */
	cy1=(int) yc[0];

	vsl_color(handle,3);		/* Set color for control pt. boxes    */

	draw_box(0,3);			/* Draw boxes around control points   */
	draw_box(1,3);
	draw_box(2,3);
	draw_box(3,3);

	draw_line(-1,3);		/* Draw convex hull (no blanking)     */

	vsl_color(handle,2);		/* Draw initial curve in different    */
	bezier();			/*   color than the boxes and lines   */

	graf_mouse(M_ON,0x0L);		/* Mousie is back and he is an...     */
	graf_mouse(ARROW,0x0L);		/*	arrow!  (Big deal)	      */

	do
	{
		vq_mouse(handle,&junk,&xx,&yy);	/* Get status of mouse        */

		for(i=0;i<4;i++)		/* Loop for each cntl. pt.    */
		{
			cx1=((int) xc[i])-xx;	/* Distance between mouse and */
			cy1=((int) yc[i])-yy;	/*  indexed c.p.	      */

			if(abs(cx1)<6 && abs(cy1)<6)  /* If in the viscinity  */
			{			      /*  of the c.p...       */
			   if(junk & 1)               /* and button is down...*/
			   {
         
         		     graf_mouse(M_OFF,0x0L);  /* No rodent...         */

                             do
			     {
 			     vq_mouse(handle,&junk,&xx1,&yy1); /* Wait 'till  */
			     } while((xx1==xx) && (yy1==yy));  /* mouse moves */
			    
			     nbs=0.05;			/* Erase hi-res curve */
			     vsl_color(handle,0);
			     bezier();

			     do
			     {	
				vq_mouse(handle,&junk,&xx,&yy);/* Track mouse */
				if((xx>6 && xx<633) || (yy>6 && yy<193))
				{			/* If in bounds...    */
				draw_line(-1,0);	/* Erase hull         */
				draw_box(i,0);		/* Erase box          */
				
				nbs=0.1;		/* Lo rez for speed   */
				vsl_color(handle,0);    /* Erase last         */
				bezier();		/* estimate           */
	

				if(xx>6 && xx<633)   /* Check x bounds        */
				{	             /* and reset control pt. */
	    				xc[i]=(float) xx;
				}	
				if(yy>6 && yy<193)
				{		     /* Ditto for y bounds    */
					yc[i]=(float) yy;
				}
				draw_line(i,3);	     /* Redraw lines w/.      */
				}		     /*   blanking	      */

				nbs=0.1;	     /* Lo rez for speed and  */
				vsl_color(handle,2); /* draw a rough          */
				bezier();	     /* estimate              */

		
			     } while(junk & 1);    /* as long as button is on */

			     draw_line(-1,3);	   /* Redraw convex hull      */

			     for(xx1=0;xx1<=3;xx1++) /* Redraw boxes          */
			     	draw_box(xx1,3);

			     nbs=0.1;			/* Lo rez and erase   */
			     vsl_color(handle,0);	/*   estimate 	      */
			     bezier();

			     nbs=0.05;			/* Hi rez and draw    */
			     vsl_color(handle,2);	/* a nice curve       */
			     bezier();

			     graf_mouse(M_ON,0x0L);     /* Mouse back on      */
			     }
		          }
		}
	}	
 	while(!(Bconstat(2)));		/* Continue until the user hits a     */
					/* key (any key...)		      */

	graf_mouse(M_OFF,0x0L);		/* Mouse off (for command processor   */
					/*   users....)		              */
}					
